"use strict";

var socket = io.connect('/', {reconnection: false});
var localMediaStream;

navigator.mediaDevices.enumerateDevices().then((mediaDevices) => {
	console.log('media devices:');
	console.log(mediaDevices);
	//PCに接続されているWebカメラ・マイクが存在することの確認
	//もしもWebカメラが接続されていなければ音声のみ取得する
	var cameraFound = !!mediaDevices.filter((mediaDevice) => {
		return mediaDevice.kind == 'videoinput';
	}).length;
	var micFound = !!mediaDevices.filter((mediaDevice) => {
		return mediaDevice.kind == 'audioinput';
	}).length;
	if(!cameraFound && !micFound){
		onFailure();
		return;
	}

	var cameraFound = !!mediaDevices.filter((mediaDevice) => {
		return mediaDevice.kind == 'videoinput';
	}).length;

	var constraints = {audio: micFound, video: cameraFound};

	//Webカメラ・マイクから映像・音声を取得する
	if(navigator.mediaDevices && navigator.mediaDevices.getUserMedia){
		//新しい方式に対応しているブラウザ用（Firefox・Chrome 53～）

		//========================================
		//リスト8「メディアの送受信」（送信・一部）
		navigator.mediaDevices.getUserMedia(constraints).then(onSuccess).catch(onFailure);
	}else{
		//旧方式にしか対応していないブラウザ用（Chrome ～52）

		//========================================
		//リスト3「メディアの送受信」（送信・一部）
		navigator.getUserMedia = navigator.getUserMedia || navigator.webkitGetUserMedia;
		navigator.getUserMedia(constraints, onSuccess, onFailure);
	}
})
.catch(onFailure);

function onSuccess(stream){
	localMediaStream = stream;
	//Webカメラ・マイクから取得したローカルメディアストリームを表示する
	//（音声のみの場合は省略）
	if(localMediaStream.getVideoTracks().length){
		var mediaElement = document.createElement('video');
		mediaElement.src = URL.createObjectURL(localMediaStream);
		// mediaElement.srcObject = localMediaStream;
		mediaElement.autoplay = true;
		mediaElement.muted = true;
		mediaElement.style.transform = 'rotateY(180deg)';
		document.querySelector('#myStream').appendChild(mediaElement);
	}
}

function onFailure(){
	document.querySelector('#container').style.display = 'none';
	alert('Webカメラ・マイクとの接続に失敗しました')
}

//シグナリングサーバに接続したとき
socket.on('connect', () => {
	console.log('Connect to signaling server');
	document.querySelector('#myId').appendChild(
		document.createTextNode(socket.id)
	);
});
//シグナリングサーバとの接続が切断したとき
socket.on('disconnect', () => {
	console.log('Disconnected from signaling server');
	document.querySelector('#container').style.display = 'none';
	alert('シグナリングサーバとの接続が切断されました');
});

//エラー処理
function onError(err){
	// 何らかのエラー処理
	console.log(err);
}