window.RTCPeerConnection = window.RTCPeerConnection || window.webkitRTCPeerConnection;
var targetId = null;
var conn = null;
var pcConstraints = {iceServers: []};

function createPeerConnection(){
	conn = new RTCPeerConnection(pcConstraints);
	//リスト3「メディアの送受信」（送信・一部）
	//（※getUserMediaはcommon/initialize.js内で実行）
	conn.addStream(localMediaStream);
	console.log(conn);

	//========================================
	//リスト2「通信経路候補の交換」（送信）ここから
	//自分の通信経路候補が見つかったらとにかく相手に教える
	conn.onicecandidate = (event) => {
		console.log(event.candidate);
		//（event.candidateが殻だったら候補探索終了の合図）
		if(event.candidate){
			socket.emit('icecandidate', targetId, event.candidate);
		}
	};
	//リスト2「通信経路候補の交換」（送信）ここまで
	//========================================

	//========================================
	//リスト3「メディアの送受信」（受信）ここから
	//相手から送られてきたMediaStreamが利用可能になったとき
	//（※自分自身のaddStreamメソッドに反応するものではない）
	conn.onaddstream = (event) => {
		console.log(event.stream);
		var elem = document.createElement('video');
		elem.src = URL.createObjectURL(event.stream);
		//↓将来的にはこれでよくなる
		//elem.srcObject = event.stream;
		elem.autoplay = true;
		document.querySelector('#stream').appendChild(elem);
	};
	//リスト3「メディアの送受信」（受信）ここまで
	//========================================

	//切断時の処理
	conn.oniceconnectionstatechange = (e) => {
		if(conn.iceConnectionState == 'disconnected'){
			ringOff();
		}
	};
	return conn;
}

function ringOff(){
	var remoteStream = document.querySelector('#stream');
	remoteStream.removeChild(remoteStream.firstElementChild);
	conn = null;
	targetId = null;
}

document.querySelector('#ringOff').addEventListener('click', ()=>{
	socket.emit('ringoff', targetId);
	ringOff();
});

//========================================
//リスト1「セッション情報の交換」 ここから
//発呼側
//==========
document.querySelector('#callingFrom').addEventListener('submit', (e) => {
	targetId = document.querySelector('#calleeId').value;
	//既に他のユーザと話中なら接続を行わない
	if(conn != null){
		e.preventDefault();
		return;
	}

	conn = createPeerConnection();
	//オファーSDPを生成
	conn.createOffer((offerSdp) => {
		//リスト4「SDP」(console.logで出力)
		console.log(offerSdp);
		console.log(offerSdp.sdp);
		//生成されたオファーSDPをローカルSDPとして保存
		conn.setLocalDescription(offerSdp, () => {
			//サーバを経由し、相手にオファーSDPを送信
			socket.emit('offer', targetId, offerSdp);
		}, onError);
	}, onError);

	e.preventDefault();
});

//着呼側
//==========
socket.on('offer', (userId, offerSdp) => {
	console.log(userId, offerSdp);
	if(conn != null){
		return;
	}
	targetId = userId;
	conn = createPeerConnection();

	//相手から送られてきたオファーSDPをリモート側SDPとして保存
	conn.setRemoteDescription(new RTCSessionDescription(offerSdp), () => {
		//アンサーSDPを生成
		conn.createAnswer((answerSdp) => {
			//生成されたアンサーSDPをローカル側SDPとして保存
			conn.setLocalDescription(answerSdp, () => {
				//相手にアンサーSDPを送信
				socket.emit('answer', targetId, answerSdp);
			}, onError);
		}, onError);
	}, onError);
});

//相手からアンサーSDPが送られてきた
socket.on('answer', (userId, answerSdp) => {
	if(userId == targetId){
		conn.setRemoteDescription(new RTCSessionDescription(answerSdp));
	}
});
//リスト1「セッション情報の交換」 ここまで
//========================================

//========================================
//リスト2「通信経路候補の交換」（受信）ここから
//相手から通信経路候補が送られてきた
socket.on('icecandidate', (userId, iceCandidate) => {
	if(userId == targetId){
		console.log(iceCandidate);
		conn.addIceCandidate(new RTCIceCandidate(iceCandidate));
	}
});
//リスト2「通信経路候補の交換」（受信）ここまで
//========================================

socket.on('ringoff', (userId) => {
	if(userId == targetId){
		ringOff();
	}
});