window.RTCPeerConnection = window.RTCPeerConnection || window.webkitRTCPeerConnection;
var targetId = null;
var conn = null;
var pcConstraints = {iceServers: []};

function createPeerConnection(){
	conn = new RTCPeerConnection(pcConstraints);
	//リスト8「メディアの送受信(Track版)」（送信・一部）
	//（※getUserMediaはcommon/initialize.js内で実行）
	//MediaStreamからMediaStreamTrackを取り出す
	localMediaStream.getTracks().forEach((track) => {
		var sender = conn.addTrack(track, localMediaStream);
		console.log(sender);
	});
	console.log(conn);

	//自分の通信経路候補が見つかったらとにかく相手に教える
	conn.onicecandidate = (event) => {
		console.log(event.candidate);
		if(event.candidate){
			socket.emit('icecandidate', targetId, event.candidate);
		}
	};

	//========================================
	//リスト8「メディアの送受信(Track版)」（受信）ここから
	//受信
	//==========
	var remoteStream = new MediaStream();
	var elem = document.createElement('video');
	elem.src = URL.createObjectURL(remoteStream);
	//↓将来的にはこれでよくなる
	//elem.srcObject = remoteStream;
	elem.autoplay = true;
	document.querySelector('#stream').appendChild(elem);
	//相手のトラックが利用可能になったとき
	conn.ontrack = (event) => {
		console.log(event);
		remoteStream.addTrack(event.track);
		//event.streamsにMediaStreamの配列が入っているので
		//これを利用してもよい（ただしontrackはTrackの数だけ
		//呼び出されるので注意）
	};
	//リスト8「メディアの送受信(Track版)」（受信）ここまで
	//========================================

	//切断時の処理
	conn.oniceconnectionstatechange = function (e){
		if(conn.iceConnectionState == 'disconnected'){
			ringOff();
		}
	};
	return conn;
}

function ringOff(){
	var remoteStream = document.querySelector('#stream');
	remoteStream.removeChild(remoteStream.firstElementChild);
	conn = null;
	targetId = null;
}

document.querySelector('#ringOff').addEventListener('click', ()=>{
	socket.emit('ringoff', targetId);
	ringOff();
});

//========================================
//リスト7「セッション情報の交換(Promise版)」 ここから
//発呼側
//==========
document.querySelector('#callingFrom').addEventListener('submit', (e) => {
	targetId = document.querySelector('#calleeId').value;
	//既に他のユーザと話中なら接続を行わない
	if(conn != null){
		e.preventDefault();
		return;
	}

	conn = createPeerConnection();
	//オファーSDPを生成
	conn.createOffer().then((offerSdp) => {
		//生成されたオファーSDPをローカル側SDPとして保存
		return conn.setLocalDescription(offerSdp);
	})
	.then(() => {
		//サーバを経由し、相手にオファーSDPを送信
		socket.emit('offer', targetId, conn.localDescription);
	}).catch(onError);
	e.preventDefault();
});

//発呼側
//==========
socket.on('offer', (userId, offerSdp) => {
	console.log(userId, offerSdp);
	if(conn != null){
		return;
	}
	targetId = userId;
	conn = createPeerConnection();

	//相手から送られてきたオファーSDPをリモート側SDPとして保存
	conn.setRemoteDescription(new RTCSessionDescription(offerSdp)).then(() => {
		//アンサーSDPを生成
		return conn.createAnswer();
	}).then((answerSdp) => {
		//生成されたアンサーSDPをローカル側SDPとして保存
		return conn.setLocalDescription(answerSdp);
	}).then(() => {
		//相手にアンサーSDPを送信
		socket.emit('answer', targetId, conn.localDescription);
	}).catch(onError);
});
//リスト7「セッション情報の交換(Promise版)」 ここまで
//========================================

socket.on('answer', (userId, answerSdp) => {
	if(userId == targetId){
		console.log(answerSdp);
		conn.setRemoteDescription(new RTCSessionDescription(answerSdp));
	}
});

socket.on('icecandidate', (userId, iceCandidate) => {
	if(userId == targetId){
		console.log(iceCandidate);
		conn.addIceCandidate(new RTCIceCandidate(iceCandidate));
	}
});

socket.on('ringoff', (userId) => {
	if(userId == targetId){
		ringOff();
	}
});